"""Version information."""
__title__ = "stylefact"
__description__ = "Statistical analysis for finance and language "
__url__ = "https://github.com/stakahashy/stylefact"
__version__ = "0.0.1"
__author__ = "Shuntaro Takahashi"
__author_email__ = "shushakura@gmail.com"
__license__ = "MIT"
__copyright__ = "Copyright 2020  Shuntaro Takahashi"
__docs_copyright__ = "2020 Shuntaro Takahashi"
