"""Module level accessible objects."""
from stylefact._version import __author__
from stylefact._version import __author_email__
from stylefact._version import __copyright__
from stylefact._version import __description__
from stylefact._version import __docs_copyright__
from stylefact._version import __license__
from stylefact._version import __title__
from stylefact._version import __url__
from stylefact._version import __version__

__all__ = (
    __version__,
    __description__,
    __url__,
    __title__,
    __author__,
    __author_email__,
    __license__,
    __copyright__,
    __docs_copyright__,
)
